/********************************************************************** 
*      Exemple d'utilisation de la structure GRAFCET 
*      pour gérer le multi-tâche
*                                     
*************************************************************************/

#include <util/atomic.h>

#include "Etape.h"



//****************************************************************************************************
const int NBMINUTERIE = 7;        // Nb de minuteries à gérer (pour chacune des bases de temps)

volatile unsigned int minuterie_1ms[NBMINUTERIE];     //Table des minuteries base de temps de 1 ms
volatile unsigned int minuterie_100ms[NBMINUTERIE];   //Table des minuteries base de temps de 100 ms
byte  compteur100ms = 100;


const int NBTACHE = 6;             // Nb de tâches à réserver

volatile boolean etatTache[NBTACHE];        //Table de l'état d'activité des tâches 
volatile byte Etape [NBTACHE];              //Table de l'étape active des tâches

typedef void (*action_t)(void);   //définir un pointeur pour les tâches


//****************************************************************************************************



//****************************************************************************************************

/******** déclaration des tâches ********/

void  Tache_init();
void  Tache_T_1();
void  Tache_T_2();
void  Tache_T_3();
void  Tache_T_4();
void  Tache_T_5();


/******** N° des tâches **************/
const unsigned int _init = 0;
const unsigned int _T_1 = 1;
const unsigned int _T_2 = 2;
const unsigned int _T_3 = 3;
const unsigned int _T_4 = 4;
const unsigned int _T_5 = 5;
/*************************************/

/**************************************** 
     Tableau de pointeurs sur les tâches
     dans l'ordre des N° des tâches 
*****************************************/

action_t Tache[] = {
    Tache_init,  
    Tache_T_1,
    Tache_T_2,
    Tache_T_3,
    Tache_T_4,
    Tache_T_5  

}; 


//****************************************************************************************************

void setup() { 
  /*------------initialise toutes les minuteries à 0 -------------*/
  
  for (int i=0; i<NBMINUTERIE;i++) {
    minuterie_1ms[i] = 0;
    minuterie_100ms[i] = 0;
    }
      
  initTimer2();   // initialise TIMER2 comme Base de Temps 

   /*------------- initialise les tâches et les étapes initiales -----------------*/
  
  for (int i =0;i<NBTACHE;i++) {      
    etatTache[i]=false;
    Etape[i]= _initX0;
  }


  etatTache[_init] = true;   // active la tâche maître
 
}




//****************************************************************************************************

void loop() {
  //=========== balayage des tâches ==============
    
   for (int i =0;i<NBTACHE;i++) {    
      if (etatTache[i]==true) Tache[i]();      
   }  
}
//****************************************************************************************************



//***************************************
//**************** LES TÂCHES ***********
//***************************************

void  Tache_init(){
  
  switch (Etape[_init]) {

    case _initX0 :

      //===== faire ici les actions transitoires =====
      Etape[_init] = _X0;     
       
    case  _X0:
      break;

    case _initX1 :
      //===== faire ici les actions transitoires =====
      Etape[_init] = _X1;     

    case _X1:
      break;

    case _initX2:
      //===== faire ici les actions transitoires =====
      Etape[_init] = _X2;     
    
    case _X2:
      break;        
  }
  
}


void  Tache_T_1(){}

void  Tache_T_2(){}

void  Tache_T_3(){}

void  Tache_T_4(){}

void  Tache_T_5(){}


//****************************************************************************************************




/********************************************************************** 
*         initialiser le timer2 en mode CTC avec interruption
**********************************************************************/

void initTimer2() 
{
  noInterrupts(); // désactiver toutes les interruptions
  TCCR2A = 0;
  TCCR2B = 0;
  TCNT2 = 0;

  TCCR2A |= (1 << WGM21);   // CTC mode
  TCCR2B |= (1 << CS22);  // 64 prescaler
  //TCCR2B |= (1 << CS21);    // 8 prescaler

  OCR2A = 250; // 16MHz/64/1000Hz => Base de Temps = 1 ms
  //OCR2A = 25; // 16MHz/64/10000Hz => Base de Temps = 100 µs

  TIMSK2 |= (1 << OCIE2A); // Activer l'IT du mode comparaison
  interrupts(); // valiser les interruptions
}


/********************************************************************** 
*                     GESTION DES MINUTERIES
**********************************************************************/

//------- Armer une minuterie avec une temporisation ---------

void armerTempo_1ms(byte numMinuterie,unsigned int Tempo){
  ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
    minuterie_1ms[numMinuterie] = Tempo;  
  }     
}

void armerTempo_100ms(byte numMinuterie,unsigned int Tempo){
  ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
    minuterie_100ms[numMinuterie] = Tempo;  
  }     
}

//-------------------------------------------------------------


//------------- Tester la fin d'une temporisation -------------

byte finTempo1ms(byte numMinuterie) {

  ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
    return (minuterie_1ms[numMinuterie]==0);  
  }  
}

byte finTempo100ms(byte numMinuterie) {

  ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
    return (minuterie_100ms[numMinuterie]==0);  
  }  
}
//-------------------------------------------------------------


/********************************************************************** 
*                      BASE DE TEMPS 1 ms
**********************************************************************/

ISR(TIMER2_COMPA_vect)  // fonction appelée sur interruption du TIMER2
{  
  /******** décrémentation des minuteries Base de Temps = 1 ms *******/
  for (byte i=0; i<NBMINUTERIE;i++) {
    if (minuterie_1ms[i] != 0) {minuterie_1ms[i] --;}
    }

    
  /******* décrémentation des minuteries Base de Temps = 100 ms ******/     
  if (compteur100ms  != 0) {
    compteur100ms--;
  }
  else {
    compteur100ms = 100;
    for (byte i=0; i<NBMINUTERIE;i++) {
     if (minuterie_100ms[i] != 0) {minuterie_100ms[i] --;}
    }
  } 
}

